
from Tkinter import *
import sys, math, traceback

# Load MP

sys.path[:0] = [ "/opt/mx/lib/mp" ]

import Mp

class MpCrystalAlignmentVideo(Frame):
	def __init__( self, master ):
		Frame.__init__( self, master )

		self.master = master
		self.video = None
		self.video_x_width = 4000.0			# pixels
		self.video_y_height = 3000.0			# pixels
		self.view_angle = 0.0				# degrees
		self.geometry_handler = None
		self.simultaneous_moves = 1
		self.one_at_a_time_update_interval = 100	# milliseconds
		self.relative_moves_list = []
		self.move_in_progress = 0

	def setup_video_display( self, video, show_cursor ):
		self.video = video

		self.video.bind("<Button-1>", self.left_button_click)

		self.video.show_cursor = show_cursor

		if ( show_cursor ):
			self.video.bind("<Button-3>", self.right_button_click)

		self.move_status = Label( self, text=' ', fg='limegreen' )

		self.video.pack(in_=self)

		self.move_status.pack(side='bottom')

	def connect_to_video( self, url, resolution ):
		self.video.connect( url, resolution )

	def continuous_video_update( self ):
#		print "============= continuous_video_update ============"
		try:
			self.video.update_image()
		except:
			print "*** Error updating video image ***"
			traceback.print_exc()

		self.after( self.video.update_interval, \
				self.continuous_video_update )

	def setup_motors( self, record_list, \
				x_name, y_name, z_name, phi_name, omega_name ):

		self.record_list = record_list

		self.x_motor = self.record_list.get_record( x_name )
		self.y_motor = self.record_list.get_record( y_name )
		self.z_motor = self.record_list.get_record( z_name )
		self.phi_motor = self.record_list.get_record( phi_name )
		self.omega_motor = self.record_list.get_record( omega_name )

	def left_button_click( self, event ):
		self.canvas_x_pixel = self.video.canvasx(event.x)
		self.canvas_y_pixel = self.video.canvasy(event.y)

		canvas_x_fraction = self.canvas_x_pixel / self.video.x_max
		canvas_y_fraction = self.canvas_y_pixel / self.video.y_max

		# For Tkinter Canvas widgets, position (0,0) is in the upper
		# left hand corner, with positive x going to the left and
		# positive y going downward.  We want to transform to a
		# coordinate system where (0,0) is in the center, with
		# positive x going to the right and positive y going _upward_.

		video_x_fraction = \
			canvas_x_fraction - self.video.x_center_fraction
		video_y_fraction = \
			self.video.y_center_fraction - canvas_y_fraction

		video_delta_x = video_x_fraction * self.video_x_width
		video_delta_y = video_y_fraction * self.video_y_height

		self.compute_relative_positions( video_delta_x, video_delta_y )

		self.move_motors()

	def compute_relative_positions( self, video_delta_x, video_delta_y ):

		if ( self.geometry_handler == None ):
			geometry_handler = default_geometry_handler
		else:
			geometry_handler = self.geometry_handler

		geometry_handler( self, video_delta_x, video_delta_y )

	def right_button_click( self, event ):
		canvas_x_pixel = self.video.canvasx(event.x)
		canvas_y_pixel = self.video.canvasy(event.y)

		canvas_x_fraction = canvas_x_pixel / self.video.x_max
		canvas_y_fraction = canvas_y_pixel / self.video.y_max

		self.video.x_center_fraction = canvas_x_fraction
		self.video.y_center_fraction = canvas_y_fraction

	def move_motors( self ):

		simultaneous_moves = int(self.simultaneous_moves)

		self.move_in_progress = 0

		if ( simultaneous_moves == 0 ):
			self.move_motors_one_at_a_time()
		else:
			self.move_motors_simultaneous()

	def move_motors_simultaneous( self ):
		# The following has not yet been fully tested. (WML)

		if ( self.move_in_progress == 0 ):

			self.move_status.configure( \
						text='Move in Progress')

			for m in self.relative_moves_list:
				motor = m[0]
				relative_move = m[1]

				try:
					motor.move_relative( relative_move )
				except:
					print \
					"*** Error starting motor %s ***" \
						% ( motor.name )
					traceback.print_exc()
		else:
			busy = 0

			for m in self.relative_moves_list:
				motor = m[0]
				motor_busy = 0

				try:
					motor_busy = motor.is_busy()
				except:
					print \
					"*** Error reading %s move status ***" \
						% ( motor.name )

				if ( motor_busy != 0 ):
					busy = 1
					break

			if ( busy == 0 ):
				self.move_in_progress = 0

			self.move_status.configure( text=' ')

		return

	def move_motors_one_at_a_time( self ):
#		print \
#  "!!!!!!!!!!!!!!!! move_motors_one_at_a_time() invoked !!!!!!!!!!!!!!!!!!!!"

		self.move_status.configure(text='Move in Progress')

		self.video.update_image()
		self.update()

		if ( self.relative_moves_list == None ):
			print \
    "ERROR ERROR ERROR: self.relative_moves_list == None ! ERROR ERROR ERROR"
			return

#		print "move_in_progress =", self.move_in_progress
#		print "relative_moves_list =",
#
#		for i in self.relative_moves_list:
#			print i[0].name, "=", i[1], ",",
#
#		print ""

		if ( len( self.relative_moves_list ) <= 0 ):
			self.move_status.configure( text = ' ' )

#			print "*** Moves complete ***"
			return

		m = self.relative_moves_list[0]

		motor = m[0]
		relative_move = m[1]

		if ( self.move_in_progress == 0 ):
			try:
				# Start the next motor.

				motor.move_relative( relative_move )
				self.move_in_progress = 1
			except:
				print "*** Error starting motor %s ***" \
					% ( motor.name )
				traceback.print_exc()
				self.update()
				return
		else:
			try:
				busy = motor.is_busy()

#				print "Motor", motor.name, ", busy =", busy

				if ( busy == 0 ):
#					print "Motor", motor.name, "move complete"

					self.move_in_progress = 0

					# Remove the motor whose move has
					# just completed from the list
					# of motors to move.

					self.relative_moves_list = \
						self.relative_moves_list[1:]
			except:
				print \
			    "*** Error reading motion status for motor %s ***" \
					% ( motor.name )
				traceback.print_exc()
				self.update()
				return

		self.after( self.one_at_a_time_update_interval, \
			self.move_motors_one_at_a_time )

		self.update()

		return

def test_geometry_handler( object, video_delta_x, video_delta_y ):
	object.relative_moves_list = [ [ object.x_motor, video_delta_x ], \
				[ object.y_motor, video_delta_y ] ]


def default_geometry_handler( object, video_delta_x, video_delta_y ):

	try:
		phi_angle = object.phi_motor.get_position()
	except:
		print "*** Error getting PHI position ***"
		traceback.print_exc()

		phi_angle = 0.0

	try:
		omega_angle = object.omega_motor.get_position()
	except:
		print "*** Error getting OMEGA position ***"
		traceback.print_exc()

		omega_angle = 0.0

	phi_angle_in_radians = phi_angle * math.pi / 180.0

	omega_angle_in_radians = omega_angle * math.pi / 180.0

	delta_x = - video_delta_y * math.sin( phi_angle_in_radians \
					- omega_angle_in_radians )

	delta_y = - video_delta_y * math.cos( phi_angle_in_radians \
					- omega_angle_in_radians )

	delta_z = video_delta_x

	object.relative_moves_list = [ [ object.z_motor, delta_z ], \
				[ object.x_motor, delta_x ], \
				[ object.y_motor, delta_y ] ]

