
from Tkinter import *
import sys, math, traceback

# Load MP

sys.path[:0] = [ "/opt/mx/lib/mp" ]

import Mp

class MpCrystalMotorDisplay(Frame):
	def __init__( self, master ):
		Frame.__init__( self, master )

		self.master = master
		self.motor_update_interval = 200
		self.display_format = "%8.4f"

	def setup_motors( self, record_list, \
				x_name, y_name, z_name, phi_name, omega_name ):

		self.record_list = record_list

		self.x_motor = self.record_list.get_record( x_name )
		self.y_motor = self.record_list.get_record( y_name )
		self.z_motor = self.record_list.get_record( z_name )
		self.phi_motor = self.record_list.get_record( phi_name )
		self.omega_motor = self.record_list.get_record( omega_name )

	def setup_motor_display( self ):

		# Now create the frame that contains the motor positions.

		self.position_frame = Frame(self)

		frame = self.position_frame

		frame.pack()

		frame.x_label = Label(frame, text='   x: ')
		frame.x_label.pack(side='left')

		frame.x_position = Label(frame, text='xxxxx', fg='blue')
		frame.x_position.pack(side='left')

		x_units       = self.x_motor.get_field('units')
		frame.x_units = Label(frame, text=x_units)
		frame.x_units.pack(side='left')

		frame.y_label = Label(frame, text='   y: ')
		frame.y_label.pack(side='left')

		frame.y_position = Label(frame, text='yyyyy', fg='blue')
		frame.y_position.pack(side='left')

		y_units       = self.y_motor.get_field('units')
		frame.y_units = Label(frame, text=y_units)
		frame.y_units.pack(side='left')

		frame.z_label = Label(frame, text='   z: ')
		frame.z_label.pack(side='left')

		frame.z_position = Label(frame, text='zzzzz', fg='blue')
		frame.z_position.pack(side='left')

		z_units       = self.z_motor.get_field('units')
		frame.z_units = Label(frame, text=z_units)
		frame.z_units.pack(side='left')

		frame.phi_label = Label(frame, text='   phi: ')
		frame.phi_label.pack(side='left')

		frame.phi_position = Label(frame, text='ppppp', fg='blue')
		frame.phi_position.pack(side='left')

		phi_units       = self.phi_motor.get_field("units")
		frame.phi_units = Label(frame, text=phi_units)
		frame.phi_units.pack(side='left')

		frame.omega_label = Label(frame, text='   omega: ')
		frame.omega_label.pack(side='left')

		frame.omega_position = Label(frame, text='ooooo', fg='blue')
		frame.omega_position.pack(side='left')

		omega_units       = self.omega_motor.get_field("units")
		frame.omega_units = Label(frame, text=omega_units)
		frame.omega_units.pack(side='left')

	def continuous_motor_update( self ):
#		print "+++++++++++++ continuous_motor_update ++++++++++++"

		self.motor_update()

		self.after( self.motor_update_interval, \
				self.continuous_motor_update )

	def motor_update( self ):
		frame = self.position_frame

		try:
			x = self.x_motor.get_position()

			x = self.display_format % ( x )

			if ( self.x_motor.is_busy() ):
				x_color = 'limegreen'
			else:
				x_color = 'blue'
		except:
			print "*** Error updating X position ***"
			traceback.print_exc
			x = "*****"
			x_color = 'red'

		frame.x_position.configure(text=x, fg=x_color)

		try:
			y = self.y_motor.get_position()

			y = self.display_format % ( y )

			if ( self.y_motor.is_busy() ):
				y_color = 'limegreen'
			else:
				y_color = 'blue'
		except:
			print "*** Error updating Y position ***"
			traceback.print_exc
			y = "*****"
			y_color = 'red'

		frame.y_position.configure(text=y, fg=y_color)

		try:
			z = self.z_motor.get_position()

			z = self.display_format % ( z )

			if ( self.z_motor.is_busy() ):
				z_color = 'limegreen'
			else:
				z_color = 'blue'
		except:
			print "*** Error updating Z position ***"
			traceback.print_exc
			z = "*****"
			z_color = 'red'

		frame.z_position.configure(text=z, fg=z_color)

		try:
			phi = self.phi_motor.get_position()

			phi = self.display_format % ( phi )

			if ( self.phi_motor.is_busy() ):
				phi_color = 'limegreen'
			else:
				phi_color = 'blue'

		except:
			print "*** Error updating PHI position ***"
			traceback.print_exc
			phi = "*****"
			phi_color = 'red'

		frame.phi_position.configure(text=phi, fg=phi_color)

		try:
			omega = self.omega_motor.get_position()

			omega = self.display_format % ( omega )

			if ( self.omega_motor.is_busy() ):
				omega_color = 'limegreen'
			else:
				omega_color = 'blue'

		except:
			print "*** Error updating OMEGA position ***"
			traceback.print_exc
			omega = "*****"
			omega_color = 'red'

		frame.omega_position.configure(text=omega, fg=omega_color)

