#! /usr/bin/env python

from Tkinter import *
import sys
import os
import Mp
import MpAxisVideo
import MpCrystalAlignmentVideo
import MpCrystalMotorDisplay

if ( len(sys.argv) != 13 ):
	print ""
	print \
"Usage: %s database x y z phi omega horiz_scale vert_scale simultaneous_flag url resolution program_name" \
			% ( sys.argv[0] )
	print ""
	sys.exit(1)

database_name = sys.argv[1]
x = sys.argv[2]
y = sys.argv[3]
z = sys.argv[4]
phi = sys.argv[5]
omega = sys.argv[6]
horiz_scale = float(sys.argv[7])
vert_scale = float(sys.argv[8])
simultaneous_flag = int(sys.argv[9])
url = sys.argv[10]
resolution = sys.argv[11]
program_name = sys.argv[12]

def destroy_callback():
#	print "Crystal alignment program exiting."
	sys.exit(0)

class Command:
	def __init__( self, func, *args, **kw ):
		self.func = func
		self.args = args
		self.kw = kw

	def __call__( self, *args, **kw ):
		args = args + self.args
		kw.update( self.kw )
		apply( self.func, args, kw )

def setup_phi_buttons( parent_frame, database, video, phi ):
	frame = Frame( parent_frame )

	phi_motor = database.get_record( phi )

	b0 = Button( frame, text=' 0 ', \
			bg='limegreen', activebackground='lightgreen' )

	b0.configure( command=Command( phi_button_callback, \
						video, phi_motor, 0 ) )

	b90 = Button( frame, text=' 90', \
			bg='limegreen', activebackground='lightgreen' )

	b90.configure( command=Command( phi_button_callback, \
						video, phi_motor, 90 ) )

	b180 = Button( frame, text='180', \
			bg='limegreen', activebackground='lightgreen' )

	b180.configure( command=Command( phi_button_callback, \
						video, phi_motor, 180 ) )

	b270 = Button( frame, text='270', \
			bg='limegreen', activebackground='lightgreen' )

	b270.configure( command=Command( phi_button_callback, \
						video, phi_motor, 270 ) )

	b0.grid(   row = 0, column = 0, sticky=N+E+S+W, padx=2, pady=2 )
	b180.grid( row = 0, column = 1, sticky=N+E+S+W, padx=2, pady=2 )
	b90.grid(  row = 1, column = 0, sticky=N+E+S+W, padx=2, pady=2 )
	b270.grid( row = 1, column = 1, sticky=N+E+S+W, padx=2, pady=2 )

	return frame

def phi_button_callback( video, phi_motor, angle ):
	current_position = phi_motor.get_position()

	relative_move = angle - current_position

	video.relative_moves_list = [ [ phi_motor, relative_move ] ]

	video.move_motors()

# ZMJ add functions to set Omega positions

def setup_omega_buttons( parent_frame, database, video, omega ):
	frame = Frame( parent_frame )

	omega_motor = database.get_record( omega )

	b_90 = Button( frame, text='-90', \
			bg='limegreen', activebackground='lightgreen' )

	b_90.configure( command=Command( omega_button_callback, \
						video, omega_motor, -90 ) )

	b0 = Button( frame, text=' 0 ', \
			bg='limegreen', activebackground='lightgreen' )

	b0.configure( command=Command( omega_button_callback, \
						video, omega_motor, 0 ) )

	b90 = Button( frame, text=' 90', \
			bg='limegreen', activebackground='lightgreen' )

	b90.configure( command=Command( omega_button_callback, \
						video, omega_motor, 90 ) )

	b_90.grid( row = 0, column = 0, sticky=N+E+S+W, padx=2, pady=2 )
	b0.grid(   row = 1, column = 0, sticky=N+E+S+W, padx=2, pady=2 )
	b90.grid(  row = 2, column = 0, sticky=N+E+S+W, padx=2, pady=2 )

	return frame

def omega_button_callback( video, omega_motor, angle ):
	current_position = omega_motor.get_position()

	relative_move = angle - current_position

	video.relative_moves_list = [ [ omega_motor, relative_move ] ]

	video.move_motors()

def setup_position_fine_adjustment( parent_frame, video, step_fraction ):

	frame = Frame( parent_frame )

	b_up = Button( frame, text='Up', \
			bg='limegreen', activebackground='lightgreen' )

	b_up.configure( command=Command( fine_callback, \
				'up', video, step_fraction) )

	b_down = Button( frame, text='Down', \
			bg='limegreen', activebackground='lightgreen' )

	b_down.configure( command=Command( fine_callback, \
				'down', video, step_fraction ) )

	b_left = Button( frame, text='Left', \
			bg='limegreen', activebackground='lightgreen' )

	b_left.configure( command=Command( fine_callback, \
				'left', video, step_fraction ) )

	b_right = Button( frame, text='Right', \
			bg='limegreen', activebackground='lightgreen' )

	b_right.configure( command=Command( fine_callback, \
				'right', video, step_fraction ) )

	b_up.grid(    row = 0, column = 1, sticky=N+E+S+W )
	b_left.grid(  row = 1, column = 0, sticky=N+E+S+W )
	b_right.grid( row = 1, column = 2, sticky=N+E+S+W )
	b_down.grid(  row = 2, column = 1, sticky=N+E+S+W )

	return frame

def fine_callback( direction, video, step_fraction ):

	y_step = step_fraction * video.video_y_height

	# We want the physical size of an X step to be the same as the
	# physical size of a Y step.

	x_step = y_step

	if ( direction == 'up' ):
		video_delta_x = 0.0
		video_delta_y = - y_step
	elif ( direction == 'down' ):
		video_delta_x = 0.0
		video_delta_y = y_step
	elif ( direction == 'left' ):
		video_delta_x = x_step
		video_delta_y = 0.0
	elif ( direction == 'right' ):
		video_delta_x = - x_step
		video_delta_y = 0.0
	else:
		error_message = "Unrecognized direction '%s'" % ( direction )
		raise NameError, error_message

	video.compute_relative_positions( video_delta_x, video_delta_y )

	video.move_motors()

def set_resolution( video, resolution ):

	resolution = video.resolution.get()

	video.connect( url, resolution )

# *** Main ***

root = Tk()

root.protocol("WM_DELETE_WINDOW", destroy_callback)

root.title( "Crystal Alignment" )

if ( os.access( database_name, os.R_OK ) == 0 ):
	print "*** Error: Cannot open MX database file", database_name, " ***"
	sys.exit(1)

database = Mp.setup_database( database_name )

# Set the MX program name.

database.set_program_name( program_name )

# Create the frames used by the program.

top_frame = Frame( root )

top_frame.pack( side='top' )

middle_frame = Frame( root )

middle_frame.pack( side='top' )

bottom_frame = Frame( root )

bottom_frame.pack( side='top' )

####

top_center_frame = Frame( top_frame )

top_center_frame.pack( side='left' )

#---

resolution_frame = Frame( top_center_frame )

resolution_frame.pack( side='top' )

resolution_label = Label( resolution_frame, text='Video Size:' )

resolution_label.pack( side='left' )

#---

crystal_video = \
	MpCrystalAlignmentVideo.MpCrystalAlignmentVideo( top_center_frame )

crystal_video.video_x_width = horiz_scale
crystal_video.video_y_height = vert_scale
crystal_video.simultaneous_moves = simultaneous_flag

crystal_video.pack( side='top', padx=10, pady=10 )

####

top_left_frame = Frame( top_frame )

top_left_frame.pack( side='left', before=top_center_frame )

phi_buttons_label = Label( top_left_frame, text='Select Phi (degrees)' )

phi_buttons_label.pack( side='top', pady=10 )

phi_buttons = setup_phi_buttons( top_left_frame, \
					database, crystal_video, phi ) 

phi_buttons.pack( side='top', padx=10 )

#### ZMJ add omega buttons under phi

omega_buttons_label = Label( top_left_frame, text='Select Omega (degrees)' )

omega_buttons_label.pack( side='top', pady=10 )

omega_buttons = setup_omega_buttons( top_left_frame, \
					database, crystal_video, omega )

omega_buttons.pack( side='top', padx=10 )

####

top_right_frame = Frame( top_frame )

top_right_frame.pack( side='left' )

fine_adjustment_label = \
		Label( top_right_frame, text="Position\nFine Adjustment" )

fine_adjustment_label.pack( side='top', pady=10 )

step_fraction = 0.01	# Fraction of the vertical picture height

position_fine_adjustment = setup_position_fine_adjustment( \
				top_right_frame, crystal_video, step_fraction )

position_fine_adjustment.pack( side='top', padx=10 )

####

crystal_motor_display = \
	MpCrystalMotorDisplay.MpCrystalMotorDisplay( middle_frame )

crystal_motor_display.pack( side='top' )

####

dismiss_button = Button( bottom_frame, text='Dismiss' )

dismiss_button.configure( command=destroy_callback )

dismiss_button.pack( pady=10 )

####

#print "Finding the motors."

crystal_video.setup_motors( database, x, y, z, phi, omega )

crystal_motor_display.setup_motors( database, x, y, z, phi, omega )

#print "Setting up the video display."

show_cursor = 1

crystal_video.setup_video_display( \
	MpAxisVideo.MpAxisVideo( crystal_video ), show_cursor )

#print "Connecting to the remote camera."

crystal_video.connect_to_video( url, resolution )

####

resolution_small = Radiobutton( resolution_frame, \
		variable=crystal_video.video.resolution, \
		text='Small', value="352x240", \
		command=Command( set_resolution, crystal_video.video, url ) )

resolution_small.pack( side='left' )

resolution_large = Radiobutton( resolution_frame, \
		variable=crystal_video.video.resolution, \
		text='Large', value="704x480", \
		command=Command( set_resolution, crystal_video.video, url ) )

resolution_large.pack( side='left' )

####

#print "Beginning GUI updates."

crystal_video.continuous_video_update()

crystal_motor_display.setup_motor_display()

crystal_motor_display.continuous_motor_update()

#print "Crystal alignment GUI is ready."

root.mainloop()

